<?php

namespace Drp\NativeEventApp\GraphQL\Resolvers;

use Drp\NativeEventApp\Auth\Facades\AuthInfoFinder;
use Drp\NativeEventApp\Models\Page;
use Drp\NativeEventApp\Models\Registration;
use Drp\NativeEventApp\Models\Segment;
use Drp\NativeEventApp\Modules\Module;
use Drp\NativeEventApp\Modules\Modules;
use Drp\NativeEventApp\Templates\Compiler;
use Illuminate\Support\Collection;

class ModuleResolver
{
    /**
     * @var \Drp\NativeEventApp\Templates\Compiler
     */
    private $compiler;

    public function __construct(Compiler $compiler)
    {
        $this->compiler = $compiler;
    }

    public function __invoke(Page $page): string
    {
        $authenticatable = AuthInfoFinder::current();
        /** @var Registration $registration */
        $registration = Registration::forEventAndAuthenticatable($page->event, $authenticatable)->first();
        $modules = $this->filterSegmentedModules(Modules::make($page->modules->all()), $registration);

        return $this->compiler->for($registration)
            ->render($modules);
    }

    private function filterSegmentedModules(Modules $modules, Registration $registration): Collection
    {
        return $modules->filter(function (Module $module) use ($registration) {
            if ($module->canHaveChildren()) {
                $module->children($this->filterSegmentedModules($module->children, $registration));
            }

            return $registration->hasAllSegments(Segment::findMany($module->viewableBy));
        });
    }
}
