<?php

namespace Drp\NativeEventApp\Notifications;

use Drp\NativeEventApp\Models\Device;
use Drp\NativeEventApp\Models\Notification as EventpalNotification;
use Drp\NativeEventApp\Notifications\Events\NotificationSendFailed;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Notifications\Notification as IlluminateNotification;
use NotificationChannels\Fcm\FcmChannel;
use NotificationChannels\Fcm\FcmMessage;
use NotificationChannels\Fcm\Resources\Notification;
use Throwable;

class PushNotification extends IlluminateNotification implements ShouldQueue
{
    use Queueable;

    /** @var string */
    protected $title;

    /** @var string */
    protected $message;

    /** @var int */
    protected $notificationId;

    /** @var string */
    protected $deviceId;

    public function __construct(string $title, string $message, int $notificationId, string $deviceId)
    {
        $this->title = $title;
        $this->message = $message;
        $this->notificationId = $notificationId;
        $this->deviceId = $deviceId;
    }

    public function toFcm(): FcmMessage
    {
        return FcmMessage::create()
            ->setNotification(
                Notification::create()
                    ->setTitle($this->title)
                    ->setBody($this->message)
            );
    }

    public function via(): array
    {
        return [FcmChannel::class];
    }

    public function failed(Throwable $exception): void
    {
        event(
            new NotificationSendFailed(
                EventpalNotification::find($this->notificationId),
                Device::find($this->deviceId),
                $exception
            )
        );
    }
}
