<?php

namespace Drp\NativeEventApp\Notifications;

use Drp\NativeEventApp\Auth\Facades\AuthInfoFinder;
use Drp\NativeEventApp\Models\Device;
use Drp\NativeEventApp\Models\Notification;
use Drp\NativeEventApp\Models\Registration;

class PushNotificationSender
{
    /**
     * @var Notification
     */
    protected $notification;

    public function __construct(Notification $notification)
    {
        $this->notification = $notification;
    }

    public function toDevice(Device $device): void
    {
        if ($device->firebase_device_id === null) {
            return;
        }

        /** @var Registration $registration */
        $registration = Registration::forEventAndAuthenticatable(
            $this->notification->event,
            AuthInfoFinder::retrieve($device->auth_identifier)
        )->first();

        $device->notify(
            new PushNotification(
                $this->notification->renderTitle($registration),
                $this->notification->renderMessage($registration),
                $this->notification->id,
                $device->id
            )
        );
    }

    public function toAllDevices(): void
    {
        Device::query()->each(function (Device $device): void {
            $this->toDevice($device);
        });
    }
}
