<?php

namespace Drp\NativeEventApp\Templates;

use Drp\NativeEventApp\Models\Registration;
use Drp\NativeEventApp\Templates\DataProviders\DataProviderRegistry;

class Compiler
{
    /**
     * @var \Drp\NativeEventApp\Templates\DataProviders\DataProviderRegistry
     */
    private $registry;

    public function __construct(DataProviderRegistry $registry)
    {
        $this->registry = $registry;
    }

    public function for(Registration $registration): self
    {
        $this->registry->for($registration);

        return $this;
    }

    public function render(string $template): string
    {
        $matches = [];
        preg_match_all('/{[A-z.\(\)\/]+}/', $template, $matches);
        $matches = collect($matches)
            ->flatten()
            ->keyBy(function ($value) {
                return $value;
            })
            ->map(function ($value) {
                $value = trim($value, '{}');

                return $this->registry->resolve($value);
            })
            ->each(function ($value, $key) use (&$template): void {
                $template = str_replace($key, $value, $template);
            });

        return $template;
    }
}
