<?php

namespace Drp\NativeEventApp\Test\Unit;

use Drp\NativeEventApp\Auth\ContactDetail;
use Drp\NativeEventApp\Auth\ContactDetails;
use Drp\NativeEventApp\Enums\AuthMethod;
use Drp\NativeEventApp\Test\TestCase;

class ContactDetailsTest extends TestCase
{
    /**
     * @test
     */
    public function only_accepts_contact_methods()
    {
        $this->expectException(\TypeError::class);
        new ContactDetails(['other']);
    }

    /**
     * @test
     */
    public function returns_if_method_set()
    {
        $details = new ContactDetails([
            new ContactDetail(AuthMethod::SMS(), '07474747474'),
        ]);
        $this->assertTrue($details->has(AuthMethod::SMS));
        $this->assertFalse($details->has(AuthMethod::EMAIL));
    }

    /**
     * @test
     */
    public function keys_by_method()
    {
        $details = new ContactDetails([
            new ContactDetail(AuthMethod::SMS(), '07474747474'),
            new ContactDetail(AuthMethod::EMAIL(), '1@1.com'),
        ]);

        $methods = $details->getIterator();

        tap($methods->get(AuthMethod::SMS), function (ContactDetail $contactDetail) {
            $this->assertEquals(AuthMethod::SMS, $contactDetail->method);
            $this->assertEquals('07474747474', $contactDetail->value);
        });

        tap($methods->get(AuthMethod::EMAIL), function (ContactDetail $contactDetail) {
            $this->assertEquals(AuthMethod::EMAIL, $contactDetail->method);
            $this->assertEquals('1@1.com', $contactDetail->value);
        });
    }

    /** @test */
    public function can_add_auth_methods()
    {
        $details = new ContactDetails();
        $details->add(AuthMethod::SMS(), '07474747474')
            ->add(AuthMethod::EMAIL(), 'test@example.com');

        $this->assertCount(2, $details);
        $this->assertTrue($details->has(AuthMethod::SMS));
        $this->assertTrue($details->has(AuthMethod::EMAIL));
    }

    /** @test */
    public function ignores_bad_contact_details()
    {
        $details = new ContactDetails();
        $details->add(AuthMethod::SMS(), 'NOT A NUMBER')
            ->add(AuthMethod::EMAIL(), 'NOT AN EMAIL ADDRESS');

        $this->assertCount(0, $details);
    }

    /** @test */
    public function can_check_for_no_contact_details()
    {
        $details = ContactDetails::make()
            ->add(AuthMethod::SMS(), 'NOT A NUMBER')
            ->add(AuthMethod::EMAIL(), 'NOT AN EMAIL ADDRESS');

        $this->assertTrue($details->isEmpty());
    }
}
