<?php

namespace Drp\NativeEventApp\Test\Unit\Notifications;

use Carbon\Carbon;
use Drp\NativeEventApp\Models\Device;
use Drp\NativeEventApp\Models\Event;
use Drp\NativeEventApp\Models\Notification;
use Drp\NativeEventApp\Notifications\Events\NotificationSendFailed;
use Drp\NativeEventApp\Notifications\PushNotification;
use Drp\NativeEventApp\Test\TestCase;
use Illuminate\Support\Facades\Event as EventFacade;
use NotificationChannels\Fcm\FcmChannel;

class PushNotificationTest extends TestCase
{
    /** @var Event */
    protected $event;

    /** @var Device */
    protected $device;

    /** @var Notification */
    protected $notification;

    /** @var PushNotification */
    protected $pushNotification;

    protected function setUp(): void
    {
        parent::setUp();

        Carbon::setTestNow('30-06-2020 13:00');

        $this->event = factory(Event::class)->create();
        $this->device = factory(Device::class)->state('firebase')->create();
        $this->notification = Notification::make($this->event, 'Title', 'Message')
            ->sendAt(now()->addHour())
            ->publish();

        $this->pushNotification = new PushNotification(
            $this->notification->title,
            $this->notification->message,
            $this->notification->id,
            $this->device->id
        );
    }

    /** @test */
    public function the_notification_failed_method_fires_a_failed_notification_event(): void
    {
        EventFacade::fake();

        $exception = new \Exception('The notification failed to send');

        $this->pushNotification->failed($exception);

        EventFacade::assertDispatched(
            NotificationSendFailed::class,
            function (NotificationSendFailed $event) use ($exception) {
                return $event->notification->id === $this->notification->id && $event->device->id === $this->device->id
                    && $event->exception->getMessage() === $exception->getMessage();
            }
        );
    }

    /** @test */
    public function it_sends_the_notification_via_the_expected_channels(): void
    {
        $expected = [FcmChannel::class];

        $this->assertEquals($expected, $this->pushNotification->via());
    }

    /** @test */
    public function it_implements_a_to_fcm_method_which_returns_a_new_message(): void
    {
        $message = $this->pushNotification->toFcm();

        $notification = $message->getNotification();

        $this->assertEquals('Title', $notification->getTitle());
        $this->assertEquals('Message', $notification->getBody());
    }
}
